#include "s08ql_lite_board.h"
#include "common.h"
#include "mtim.h"
#include "printf.h"
#include "wdog.h"
#include "adc.h"
#include "uart.h"
#include "ics.h"
#include "mtim.h"
#include "tpm.h"


/******************************************************************************
* Global functions
******************************************************************************/
void Button_Init(void);
void configure_LEDs_for_GPIO(void);
void configure_LEDs_for_PWM ();

extern uint8_t g_delay_1ms_done; 
extern uint8_t button2IsPressedFlag, button3IsPressedFlag;

//#define MAX_PWM_VALUE 0

void delay()
{
    uint8_t n;
    
    for(n=0;n<50;n++) 
    {
        asm(nop);
    }
}

void DelayMS(uint16_t timeMS)
{
    uint16_t  delay_given_ms = 0;
    g_delay_1ms_done = 0;

    MTIM_Start();
    
    while (delay_given_ms < timeMS)
    {
        while (!g_delay_1ms_done);
        g_delay_1ms_done = 0;
        delay_given_ms++;
    }

    MTIM_Stop();
}


void BOARD_InitDebugConsole(void)
{ 
	Init_SCI(BUS_CLK_HZ); 
}

void Sys_Init(void)
{
    /* Disable the watchdog timer */
    /* WARNINGIN: DO NOT single step this function call, otherwise
    * will cause watch dog reset
    */
    //SOPT1 &= ~SOPT1_COPE_MASK;
	//SOPT1 |= SOPT1_RSTPE_MASK;
	SOPT1 = 0x3;			//SOPT1 is write-once register, enable BKGD/RESET pin.
    /*
    /////////////////////////////////////////////////////////////////////////////////////////
    // Peripheral Init
    // --------------------------------------------------------------------------------------
    /////////////////////////////////////////////////////////////////////////////////////////
    */

    /* output bus clock if enabled */
#if defined(USE_FEE)  
    // Initialize system clock to 16MHz with external 8M crystal
    FEI_to_FEE();
#endif  



    /* enable stop,*/
    //SYS_SOPT1 = 0x4d;
    /* Configure SCI for printing */                       
    BOARD_InitDebugConsole();
    

#if	defined(PRINT_SYS_LOG)
    print_sys_log();
#endif



    /* _asm CLI */ /* enable interrupts */
}



void BOARD_InitLED(void)
{
	
  // LED_ConfigAsGPIO();
}

void BOARD_InitPins(void)
{
	//BOARD_InitLED();
	
	
}

void LED_Init(void)
{
    // 5 LEDs are connected to PTE7, PTG0/1/2/3, PTH2/6/7
    
    // configure the pin as GPIO output
    LED0_Init();
    LED1_Init();
    LED2_Init();
    LED3_Init();
    LED4_Init();
    
    
    LED0_Off();
    LED1_Off();
    LED2_Off();
    LED3_Off();
    LED4_Off();

}

void LED_Test(void)
{

    LED4_On();
    DelayMS(100);
    LED4_Off();
    DelayMS(100);
    LED4_On();
    DelayMS(100);
    LED4_Off();    
    DelayMS(100);
    LED4_On();
    DelayMS(100);


    LED4_Off(); 
    DelayMS(100);
    LED4_On();
    DelayMS(100);
    LED4_Off();
    DelayMS(100);
    LED4_On();
    DelayMS(100);
    LED4_Off();
    DelayMS(100);

}
/*****************************************************************************//*!
+FUNCTION----------------------------------------------------------------
* @function name: Button_Init
*
* @brief initialize button switches.
*        
* @param  none 
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void Button_Init(void)	
{	
    // 2 mechanical buttons are connected to PTB2/3
	SCGC2 |= SCGC2_KBI_MASK;
    
	KBISC_KBIE = 0;       // disable the KBI interrupt    KBI1_SC_KBMOD = 0;      //KBI detects edges only
    KBIES &= (~KBIES_KBEDG2_MASK) & (~KBIES_KBEDG3_MASK);     //select the KBI polarity: falling edge
    //PORT_PTDPE = PORT_PTDPE_PTDPE4_MASK | PORT_PTDPE_PTDPE5_MASK;	// enable internal pullup
    KBIPE = KBIPE_KBIPE6_MASK | KBIPE_KBIPE7_MASK;                    // enable KBI pin
    KBISC_KBACK = 1;	    //clear any false interrupt
    KBISC_KBIE  = 1;      //enable the KBI interrupt
}

void Button_Test(void)
{

    printf("Button test: BEGINS\n");
    
    button2IsPressedFlag = 0;
    button3IsPressedFlag = 0;

    printf("Button test: press button 2 or 3 on board now\n");
    printf("Button test: pass test until button 2, 3 are pressed\n");
    
    while (1)
    {
    	//LED4_Off();
        //waiting for both button 2, 3 are pressed
        if ((button2IsPressedFlag) && (button3IsPressedFlag))
        {
            printf("Button test: PASSED\n");
            printf("\n");
            return;
        }
    }
}


void DbgConsole_Init(void)
{
    // OSBDM serial outputs are connected to SCI2 PTD6/7

    Init_SCI(BUS_CLK_HZ);   // init SCI2
}


void AnalogInput_Init(void)
{
    //init potentiometer, thermistor
    //potentiometer is connected to ADC: PTF4/ADP12
    // thermistor is connected to ADC: PTF5/ADP13, PTF6/ADP14
    
    //ADC_Init();
}

void AnalogCompare_Init(void)
{
    // CMP is connected to ACMP: PTA1/ACMP1
    // compare source can be selected from potentiometer or DAC5571

    //CMP_Init();
}

void Buzzer_Init(void)
{
    // buzzer is connected to TPM: PTA0/TPMCH0
    TPM_Init();
}

void Buzzer_On(void)
{
    TPM_Start();
}

void Buzzer_Off(void)
{
    TPM_Stop();
}

void Buzzer_Beep(void)
{
    Buzzer_On();
    DelayMS(10);
    Buzzer_Off();
}

void Buzzer_Test(void)
{
    Buzzer_On();
    DelayMS(100);
    Buzzer_Off();
}
/************************************************************************
*       configure_LEDs_for_PWM - Set up TPM1 and TPM2                   *
*************************************************************************
; Since the LEDs are connected to the Timer/Pulse Width Modulator modules (TPMs),
; we can configure the TPMs to generate an edge-aligned PWM signal. 
*/

/************************************************************************
*       configure_LEDs_for_GPIO - Stop TPMs from driving LEDs           *
*************************************************************************
; Modifies the TPM channel configurations so that the TPMs no longer drive
; the LEDs. Control over the LEDs reverts to the GPIO method
*/

/************************************************************************
*       leds_all_on - Turn all LEDs on                                  *
*************************************************************************/


/************************************************************************
*       leds_all_off - Turn all LEDs off                                *
*************************************************************************/


/************************************************************************
*       leds_cross - Turn on LEDs to form a "x" cross                   *
*************************************************************************
*/

/************************************************************************
*       leds_plus - Turn on LEDs to form a "+" plus sign                *
*************************************************************************
*/

/*
 * end here
 */
