/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file adc.c
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief providing APIs for configuring analog-to-digital converter (ADC). 
*
*******************************************************************************
*
* provide ADC driver,complete potentiometer and thermistor test 
******************************************************************************/
#include "common.h"
#include "printf.h"
#include "adc.h"
#include "stdio.h"
#include "stdlib.h"
/******************************************************************************
* Local function
******************************************************************************/
unsigned short ADC_GetResult( unsigned char channel);

/******************************************************************************
* Local variables
******************************************************************************/
unsigned short m_uiADC_ResultBuff[10];
unsigned char  m_ucADC_BuffCount;
/******************************************************************************
* Local function prototypes
******************************************************************************/
/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: ADC_Init
   *
   * @brief initialize ADC as 12 bit mode, software trigger,enable ADC0
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_Init( void )
{
	//APCTL1 = 0x01;          //disable IO on ADC CH0	
	SCGC1 |= SCGC1_ADC_MASK;
	APCTL1 = 0x00;
	
	ADCCFG_ADICLK = CLOCK_SOURCE_BUS_CLOCK;
	ADCCFG_ADIV   = ADC_ADIV_DIVIDE_4;
	ADCCFG_MODE   = ADC_MODE_12BIT;
	ADCCFG_ADLSMP = 1;
	
	m_ucADC_BuffCount = 0;
}
/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: ADC_GetResult
   *
   * @brief start a conversion and get conversion result
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
unsigned short ADC_GetResult( unsigned char channel)
{
	ADCSC1 = channel; // initial the ADC conversion,select ADC channel
	while( !ADCSC1_COCO );
	return ADCR;
}


void Potentiometer_Init(void)
{
    uint8_t temp;

    if (POTENTIO_ADC_CHANNEL0<8)
    {
        temp = APCTL1;
        temp |= (1<<(POTENTIO_ADC_CHANNEL0%8));
        APCTL1 = temp;
   }
    if (POTENTIO_ADC_CHANNEL1<8)
    {
        temp = APCTL1;
        temp |= (1<<(POTENTIO_ADC_CHANNEL1%8));
        APCTL1 = temp;
   }

    //ADC_APCTL2 |= 0x10;          //disable IO on ADC CH12, connected to the potentiometer
}
/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: ADC_test
   *
   * @brief sample potentiometer voltage and switch LED indication according to ADC result
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
unsigned short Potentiometer_GetValue( unsigned char ADC_CHANNEL)
{
    unsigned short uiADC_Value;
    unsigned long uiADC_Value_temp;

    unsigned char i = 0;   
    
	uiADC_Value = ADC_GetResult(ADC_CHANNEL);

    for (i=0; i<5; i++)
    {
        uiADC_Value_temp = ADC_GetResult(ADC_CHANNEL);

        uiADC_Value_temp += uiADC_Value;

        uiADC_Value = (unsigned short)(uiADC_Value_temp/2);
    }
    
	return uiADC_Value;
}

void Potentiometer0_Test(void)
{
    unsigned short uiADC_Value0, uiADC_Value1, uiADC_Value2;
    unsigned long uiADC_Value_temp;

    unsigned char i = 0;

    printf("Potentiometer0 test: BEGINS\n");

      
    while(1)
    {
        DelayMS(1);

        uiADC_Value0 = Potentiometer_GetValue(POTENTIO_ADC_CHANNEL0);

        if (uiADC_Value0 <= 0xFFF)
        {
            //uiADC_Value is valid, as ADC is configured as 12-bit mode
            printf("Potentiometer0 test: turn the Potentiometer R39 on board now\n");
            printf("Potentiometer0 test: pass test until the Potentiometer R39 is turned to a different value\n");

            while (1)
            {
                //ADC measures Potentiometer value
            	uiADC_Value1 = Potentiometer_GetValue(POTENTIO_ADC_CHANNEL0); 
            	
                //printf ("uiADC_Value1 = 0x%x\n", uiADC_Value);
            	
            	uiADC_Value2 = abs(uiADC_Value1-uiADC_Value0); 
            	
                //check the ADC value changes when the Potentiometer is turned
                if (uiADC_Value2 > (0xFFF/8)*2)
                {
                    
                    LED4_On();       
                    DelayMS(500);                
                    LED4_Off();
                  
                    printf ("Potentiometer0 test: PASSED\n");
                    printf("\n");
                    return;
                }
            } 
        }
        else
        {            
            //uiADC_Value is invalid
            printf ("Potentiometer0 test: FAILED\n");
            while (1);
        }
    }
}

void Potentiometer1_Test(void)
{
    unsigned short uiADC_Value0, uiADC_Value1, uiADC_Value2;
    unsigned long uiADC_Value_temp;

    unsigned char i = 0;

    printf("Potentiometer1 test: BEGINS\n");

      
    while(1)
    {
        DelayMS(1);

        uiADC_Value0 = Potentiometer_GetValue(POTENTIO_ADC_CHANNEL1);

        if (uiADC_Value0 <= 0xFFF)
        {
            //uiADC_Value is valid, as ADC is configured as 12-bit mode
            printf("Potentiometer1 test: turn the Potentiometer R38 on board now\n");
            printf("Potentiometer1 test: pass test until the Potentiometer R38 is turned to a different value\n");

            while (1)
            {
                //ADC measures Potentiometer value
            	uiADC_Value1 = Potentiometer_GetValue(POTENTIO_ADC_CHANNEL1); 
            	
                //printf ("uiADC_Value = 0x%x\n", uiADC_Value);
            	
            	uiADC_Value2 = abs(uiADC_Value1-uiADC_Value0); 
            	
                //check the ADC value changes when the Potentiometer is turned
                if (uiADC_Value2 > (0xFFF/8)*2)
                {
                    
                    LED4_On();       
                    DelayMS(500);                
                    LED4_Off();
                  
                    printf ("Potentiometer1 test: PASSED\n");
                    printf("\n");
                    return;
                }
            } 
        }
        else
        {            
            //uiADC_Value is invalid
            printf ("Potentiometer1 test: FAILED\n");
            while (1);
        }
    }
}





void Thermistor_Init(void)
{
    uint8_t temp;

    if (THERM_ADC_CHANNEL0<8)
    {
        temp = APCTL1;
        temp |= (1<<(THERM_ADC_CHANNEL0%8));             //disable IO on ADC CH2, connected to the Thermistor
        APCTL1 = temp;
   }
   
    if (THERM_ADC_CHANNEL1<8)
    {
        temp = APCTL1;
        temp |= (1<<(THERM_ADC_CHANNEL1%8));           //disable IO on ADC CH3, connected to the Thermistor
        APCTL1 = temp;
   }
  
}

void Thermistor_Test( void )
{
    unsigned short uiADC_Value0, uiADC_Value1, uiADC_ValueBaseline, uiADC_Value;
    
    unsigned long uiADC_Value0_temp, uiADC_Value1_temp;

    unsigned char i = 0;

    //printf("Thermistor test: BEGINS\n");
    
    LED4_Off();

    while(1)
    {

        DelayMS(1);

        uiADC_Value0 = ADC_GetResult(THERM_ADC_CHANNEL0);
        uiADC_Value1 = ADC_GetResult(THERM_ADC_CHANNEL1);

        for (i=0; i<10; i++)
        {
            uiADC_Value0_temp = ADC_GetResult(THERM_ADC_CHANNEL0);
            uiADC_Value1_temp = ADC_GetResult(THERM_ADC_CHANNEL1);

            uiADC_Value0_temp += uiADC_Value0;
            uiADC_Value1_temp += uiADC_Value1;

            uiADC_Value0 = (unsigned short)(uiADC_Value0_temp/2);
            uiADC_Value1 = (unsigned short)(uiADC_Value1_temp/2);
        }

        uiADC_ValueBaseline = uiADC_Value0-uiADC_Value1;

        if (uiADC_ValueBaseline > 0x100)
        {
            //uiADC_ValueBaseline is valid

            printf("Thermistor test: heat the Thermistor RT1 on board now\n");
            printf("Thermistor test: pass test until the Thermistor is heated over the given value\n");
            
            while (1)
            {
                //ADC measures thermister value
                uiADC_Value0_temp = ADC_GetResult(THERM_ADC_CHANNEL0);
                uiADC_Value1_temp = ADC_GetResult(THERM_ADC_CHANNEL1);

                //calculate the average ADC value
                 uiADC_Value0_temp += uiADC_Value0;
                uiADC_Value1_temp += uiADC_Value1;
                uiADC_Value0 = (unsigned short)(uiADC_Value0_temp/2);
                uiADC_Value1 = (unsigned short)(uiADC_Value1_temp/2);
                
                uiADC_Value = uiADC_Value0-uiADC_Value1;

                //printf ("uiADC_Value = 0x%x\n", uiADC_Value);

                //check the ADC value changes when the thermistor is heated
                if (uiADC_Value < uiADC_ValueBaseline -0x20)
                {
                
                    LED4_On();             
                    DelayMS(300);
                    LED4_Off(); 
                    
                    printf ("Thermistor test: PASSED\n");
                    printf("\n");
                    return;
                }
            }
            
        }

        else
        {            
            //uiADC_ValueBaseline is invalid
            printf ("Thermistor test: FAILED\n");
            while (1);
        }
    }
    
}
	
