/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file uart.c
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief providing APIs for configuring serial communications interface (SCI). 
*
*******************************************************************************
*
* provide APIs for access SCIs. more APIs to be added.
******************************************************************************/
#include "uart.h"
#include "common.h"

  /*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: TERMIO_PutChar3
   *
   * @brief send a char thru SCI2
   *        
   * @param  send, a char to be sent
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/

  void TERMIO_PutChar(char send)
  {
    char dummy;

    while(!( SCIS1 & SCIS1_TDRE_MASK));
    dummy = SCIS1;
    SCID  = send;
  }//end SCI_PutChar

  /*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: TERMIO_GetChar3
   *
   * @brief receives a char from SCI2
   *        
   * @param  none
   *
   * @return a char received
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/

  char TERMIO_GetChar(void)
  {
    char dummy;

    while(( SCIS1 & SCIS1_RDRF_MASK) !=SCIS1_RDRF_MASK);
    dummy = SCIS1;
    dummy = SCID;
    return dummy;
  } //end SCI_GetChar

  /*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: Init_SCI2
   *
   * @brief initializes SCI2 baud rate, enable both tx and rx
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/

void Init_SCI(dword busCLKHz)
{
    //SCIx_BD = BUSCLK/16/baudrate
    /* make sure clocks to peripheral modules are enabled */
    SCGC1 |= SCGC1_SCI_MASK;

    SCIBD = 54; //16.8MHz/2/16/115200
    SCIC1  = 0;     // 8bit mode, 1 stop bit, no parity
    SCIC2 |= SCIC2_TE_MASK|SCIC2_RE_MASK;  // enable TX , enable RX
}

  
  /*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: wait_sci_tx_empty
   *
   * @brief loops to wait for the SCI tx empty
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/

void wait_sci_tx_empty(void){
	
    while(( SCIS1 & SCIS1_TDRE_MASK) !=SCIS1_TDRE_MASK);

}
