#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "s08ql_lite_board.h"
#include "common.h"
#include "string.h"
#include "math.h"
#include "printf.h"
#include "uart.h"
#include "mtim.h"
#include "adc.h"
#include "tpm.h"
#include "acmp.h"
#include "rtc.h"

unsigned char buttonStates = 0;

unsigned short g_delay_ms = 0;
uint8_t g_delay_1ms_done = 0;


#define Button2_Is_Pressed()	!(buttonStates & (1<<2))
#define Button3_Is_Pressed()	!(buttonStates & (1<<3))

uint8_t button2IsPressedFlag = 0;
uint8_t button3IsPressedFlag = 0;



void main(void) {

	Sys_Init();
	EnableInterrupts;
  /* include your code here */
	
    printf("\n-- S08QL-LITE FAT BEGINS --\n");
    printf("\n");
  
    //Initialize MTIM for delay   
    MTIM_Init();  
    
    //Initialize TPM TPMCH0 used for buzzer
    Buzzer_Init();
    //Test Buzzer, turn on/off
    Buzzer_Test();
    
    //Initialize GPIO used for LED
    LED_Init();
    //Test LED4, the LED will be toggle     
    LED_Test();
    Buzzer_Beep();
    
    //Initialize ADC, 12bit mode
    ADC_Init();   
    //Initialize ADC ADP2/3 user for thermistor measurement
    Thermistor_Init();    
    Thermistor_Test();
    Buzzer_Beep();

    //Initialize ADC ADP0 ADP1 user for potentiometer measurement
    Potentiometer_Init();
    //potentiometer R39
    Potentiometer0_Test(); 
    Buzzer_Beep();
    //potentiometer R38     
    Potentiometer1_Test();
    Buzzer_Beep();
     
    //Initialize GPIO PTD4/5 used for KBI SW2/SW3
    Button_Init();
    Button_Test();
    Buzzer_Beep();     
      
    //Initialize RTC  
    RTC_Init();
    RTC_Start(); 
      
    printf("\n-- S08QL-LITE FAT END --\n");
    printf("\n");   
    while(1); 
  /* loop forever */
  /* please make sure that you never leave main */
}



//MTIM is used for delay 1 milliseconds
interrupt VectorNumber_Vmtim void Mtim_ISR(void)
 {
     if(MTIMSC_TOF)
     {
         // clear the flag
         MTIMSC_TOF = 0;	
     }
     g_delay_1ms_done = 1;
     //LED4_Toggle();
 };  

interrupt VectorNumber_Vkeyboard void Button_ISR(void)
{
    KBISC_KBACK = 1;                       //clear the KBI interrupt flag
    //KBI1_PE = 0;                                // NOTE: disable KBI pin function to avoid the mechanical jitters

    KBIPE &= (~KBIPE_KBIPE6_MASK) & (~KBIPE_KBIPE7_MASK);                    // disable KBI pin
    PTBDD &= ~((1<<2)|(1<<3));    //enable gpio input
    asm(nop);
    asm(nop);
    buttonStates = PTBD;         //read PORT data register to recognize which button is pushed

    if(Button2_Is_Pressed())
    {
    	//LED4_On();  	
        button2IsPressedFlag = 1;
        printf("SW2 is pressed\n");
    }
    else if(Button3_Is_Pressed())
    {
    	//LED4_Off();
        button3IsPressedFlag = 1;
        printf("SW3 is pressed\n");        
    }

    // restore KBI pin function after PORT data register
    KBIPE = KBIPE_KBIPE6_MASK | KBIPE_KBIPE7_MASK; /* enable KBI pin */
    KBISC_KBACK = 1;	
}


interrupt VectorNumber_Vacmp void  ACMP_Int_ISR(void)
{
  
    ACMPSC_ACF = 0;
    if (ACMPSC_ACO)
    {
        LED4_On();
    }
    else
    {
        LED4_Off();     
    }
}

//RTC 1s interrupt
interrupt VectorNumber_Vrtc void  RTC_ISR(void)
{
    
	if(RTCSC_RTIF)  // clear the flag
	{
            RTCSC_RTIF = 1;	
            LED4_Toggle();          
	}
			
}

